@@ -13,10 +13,10 @@ void JOINT_gp(int *n, int *T, int *r, int *rT, int *p, int *N,
      int *cov, int *spdecay, double *shape_e, double *shape_eta,
      double *prior_a, double *prior_b, double *prior_mubeta, 
      double *prior_sigbeta, double *prior_omu, double *prior_osig,
-     double *phi, double *tau, double *phis, int *phik,
+     double *phi, double *tau, double *phis, int *phik, double *nu,
      double *d, double *sig_e, double *sig_eta, 
      double *beta, double *X, double *z, double *o, int *constant,
-     double *phip, double *accept, double *sig_ep, double *sig_etap, 
+     double *phip, double *accept, double *nup, double *sig_ep, double *sig_etap, 
      double *betap, double *op)
 {     
      int n1, nn, r1, p1, rn, N1, col; 
@@ -35,20 +35,33 @@ void JOINT_gp(int *n, int *T, int *r, int *rT, int *p, int *N,
    det = (double *) malloc((size_t)((1)*sizeof(double)));   
    S = (double *) malloc((size_t)((n1*n1)*sizeof(double)));   
 // Rprintf("   phi: %4.4f, cov: %i\n", phi[0], cov[0]);         
-   covFormat(cov, n, phi, d, sig_eta, S, det, Sinv, Qeta);   
+   covFormat(cov, n, phi, nu, d, sig_eta, S, det, Sinv, Qeta);   
    MProd(beta, constant, p, X, N, XB);
+// check nu
+   if(cov[0]==4){
+      nu_gp_DIS(cov, Qeta, det, phi, nu, n, r, T, rT, N, d, sig_eta, XB, o, 
+      constant, nup);
+//      Rprintf("   nu: %4.4f, nup: %4.4f \n", nu[0], nup[0]);
+//      covFormat(cov, n, phi, nup, d, sig_eta, S, det, Sinv, Qeta);   
+   }
+   else {
+      nup[0] = nu[0];
+   }  
+
 // fixed values for phi 
    if(spdecay[0] == 1){
     accept[0] =0.0;
     phip[0] = phi[0];
+    covFormat(cov, n, phip, nup, d, sig_eta, S, det, Sinv, Qeta);   
    }
 // discrete sampling for phi 
    else if(spdecay[0] == 2){
-     phi_gp_DIS(cov, Qeta, det, phi, phis, phik, n, r, T, rT, N, 
+     phi_gp_DIS(cov, Qeta, det, phi, phis, phik, nup, n, r, T, rT, N, 
      prior_a, prior_b, d, sig_eta, XB, o, constant, accept, phip);
-     if(accept[0] == 1.0){    
-     covFormat(cov, n, phip, d, sig_eta, S, det, Sinv, Qeta);   
-     }
+     covFormat(cov, n, phip, nup, d, sig_eta, S, det, Sinv, Qeta);   
+//       if(accept[0] == 1.0){    
+//        covFormat(cov, n, phip, nu, d, sig_eta, S, det, Sinv, Qeta);   
+//       }
    }
 // Random-Walk MH-within-Gibbs sampling for phi
    else if(spdecay[0] == 3){
@@ -64,16 +77,16 @@ void JOINT_gp(int *n, int *T, int *r, int *rT, int *p, int *N,
       tmp[0] = -log(phi[0]); 
 // Rprintf("   phi: %4.4f, tmp: %4.4f, cov: %i\n", phi[0], tmp[0], cov[0]);      
       mvrnormal(constant, tmp, tau, constant, phi2);
-      phi2[0]= exp(-phi2[0]);
+      phi2[0]= exp(-phi2[0]);      
 // Rprintf("   phi: %4.4f, tmp: %4.4f, cov: %i\n", phi[0], tmp[0], cov[0]);      
-      covFormat(cov, n, phi2, d, sig_eta, S, det2, Sinv, Qeta2);   
+      covFormat(cov, n, phi2, nup, d, sig_eta, S, det2, Sinv, Qeta2);   
 // Rprintf("   phi: %4.4f, phi2: %4.4f, cov: %i\n", phi[0], phi2[0], cov[0]);
 //     randow-walk M  
      phi_gp_MH(Qeta, Qeta2, det, det2, phi, phi2, n, r, T, rT, N, 
      prior_a, prior_b, XB, o, constant, accept, phip);
 // Rprintf("   phi: %4.4f, phi2: %4.4f, cov: %i\n", phi[0], phi2[0], cov[0]);
      if(accept[0] == 1.0){    
-     covFormat(cov, n, phip, d, sig_eta, S, det, Sinv, Qeta);   
+         covFormat(cov, n, phip, nup, d, sig_eta, S, det, Sinv, Qeta);   
      }
      free(Qeta2); free(det2); free(tmp); free(phi2);
    }
@@ -81,15 +94,15 @@ void JOINT_gp(int *n, int *T, int *r, int *rT, int *p, int *N,
      //;
 //     exit(9);
    }   
-
    beta_gp(n, r, T, rT, p, prior_mubeta, prior_sigbeta, Qeta, X, o,
    constant, betap);
    MProd(betap, constant, p, X, N, XB);
-   sig_e_gp(n, r, T, rT, N, shape_e, prior_b, o, z, constant, sig_ep);
+   sig_e_gp(n, r, T, rT, N, shape_e, prior_b, XB, o, z, sig_e, constant, sig_ep);
    sig_eta_gp(n, r, T, rT, shape_eta, prior_b, Sinv, XB, o, constant, sig_etap);
-   o_gp(n, r, T, rT, p, prior_omu, prior_osig, sig_ep, sig_etap, S, Qeta, 
+   o_gp(n, r, T, rT, p, prior_omu, prior_osig, sig_e, sig_etap, S, Qeta, 
    XB, z, constant, op);     
-        
+
+           
    free(Qeta); free(XB); free(Sinv); free(det); free(S); 
    return;
 }
@@ -97,7 +110,8 @@ void JOINT_gp(int *n, int *T, int *r, int *rT, int *p, int *N,
 
 // Posterior distribution for "sig_e"
 void sig_e_gp(int *n, int *r, int *T, int *rT, int *N, double *shape, 
-     double *prior_b, double *o, double *z, int *constant, double *sig2e)
+     double *prior_b, double *XB, double *o, double *z, double *sig_e,
+     int *constant, double *sig2e)
 {
      int i, t, l, n1, r1, T1, col;
      n1 =*n;
@@ -105,12 +119,14 @@ void sig_e_gp(int *n, int *r, int *T, int *rT, int *N, double *shape,
      T1 =*T;
      col =*constant;
      
-     double *z1, *o1, *zo, *zzoo;
+     double *z1, *o1, *zo, *zzoo, *XB1, *tmp;
      z1 = (double *) malloc((size_t)((n1*col)*sizeof(double)));
      o1 = (double *) malloc((size_t)((n1*col)*sizeof(double)));
      zo = (double *) malloc((size_t)((n1*col)*sizeof(double)));
      zzoo = (double *) malloc((size_t)((col)*sizeof(double)));
-     
+     XB1 = (double *) malloc((size_t)((n1*col)*sizeof(double)));     
+     tmp = (double *) malloc((size_t)((col)*sizeof(double)));     
+          
      double u, v, b, sige[1];
      u = 0.0;
      v = 0.0;
@@ -119,8 +135,14 @@ void sig_e_gp(int *n, int *r, int *T, int *rT, int *N, double *shape,
         for(t=0; t<T1; t++){
              extract_alt2(l, t, n, rT, T, o, o1);
              extract_alt2(l, t, n, rT, T, z, z1);
+             extract_alt2(l, t, n, rT, T, XB, XB1);
              for(i=0; i<n1; i++){
-                 zo[i] = z1[i]-o1[i];
+//                 zzoo[0] = o1[i]-XB1[i];
+                 zzoo[0] = z1[i]-o1[i];
+                 tmp[0] = 0.005;
+                 mvrnormal(constant, zzoo, tmp, constant, zzoo);                              
+//                 zo[i] = z1[i]-(o1[i]+zzoo[0]);
+                 zo[i] = zzoo[0];
              }
              MProd(zo, constant, n, zo, constant, zzoo);
              u += zzoo[0];          
@@ -132,7 +154,7 @@ void sig_e_gp(int *n, int *r, int *T, int *rT, int *N, double *shape,
      sige[0] = rigammaa(v, u);
      *sig2e = sige[0];
 
-     free(z1); free(o1); free(zo); free(zzoo);
+     free(z1); free(o1); free(zo); free(zzoo); free(XB1);free(tmp);
      return;                  
 }     
 
@@ -196,7 +218,7 @@ void beta_gp(int *n, int *r, int *T, int *rT, int *p, double *prior_mu,
      col =*constant;
      
      
-     double *del, *chi, *ot1, *X1, *tX1, *out, *tX1QX1, *tX1Qo, *det, *mu;
+     double *del, *chi, *ot1, *X1, *tX1, *out, *tX1QX1, *tX1Qo, *det, *mu, *I;
      del = (double *) malloc((size_t)((p1*p1)*sizeof(double)));
      chi = (double *) malloc((size_t)((p1*col)*sizeof(double)));     
      ot1 = (double *) malloc((size_t)((n1*col)*sizeof(double)));
@@ -207,7 +229,7 @@ void beta_gp(int *n, int *r, int *T, int *rT, int *p, double *prior_mu,
      tX1Qo = (double *) malloc((size_t)((p1*col)*sizeof(double)));
      det = (double *) malloc((size_t)((col)*sizeof(double)));
      mu = (double *) malloc((size_t)((p1*col)*sizeof(double)));     
-                    
+     I = (double *) malloc((size_t)((p1*p1)*sizeof(double)));                         
      
      for(i=0; i<p1*p1; i++){
            del[i] = 0.0;
@@ -215,7 +237,7 @@ void beta_gp(int *n, int *r, int *T, int *rT, int *p, double *prior_mu,
      for(i=0; i<p1; i++){
            chi[i] = 0.0;
      }   
-     
+ 
      for(l=0; l<r1; l++){
      for(t=0; t<T1; t++){
           extract_X(t, l, n, r, T, p, X, X1);    // n x p
@@ -232,9 +254,11 @@ void beta_gp(int *n, int *r, int *T, int *rT, int *p, double *prior_mu,
      }
      }
 
+     IdentityM(p, I);
      for(i=0; i<p1*p1; i++){
-     del[i] = del[i] + 1.0/prior_sig[0];
+     del[i] = del[i] + I[i]*(1.0/prior_sig[0]);     
      }
+     free(I); 
      for(i=0; i<p1; i++){
      chi[i] = chi[i] + prior_mu[0]/prior_sig[0];
      }
@@ -387,7 +411,7 @@ void phi_gp_MH(double *Qeta1, double *Qeta2, double *det1, double *det2,
      else{    
      tr1 = (a-1.0)*log(phi1[0])-b*phi1[0]-0.5*rT1*log(det1[0])- u; 
      tr2 = (a-1.0)*log(phi2[0])-b*phi2[0]-0.5*rT1*log(det2[0])- v; 
-     ratio[0] = exp(tr2 - tr1);
+     ratio[0] = exp(tr2 + exp(tr2) - tr1 - exp(tr1));
      ratio_fnc(ratio, constant, U);
      if(U[0] < ratio[0]){
           phip[0] = phi2[0];
@@ -406,8 +430,8 @@ void phi_gp_MH(double *Qeta1, double *Qeta2, double *det1, double *det2,
 
 
 // Discrete sampling for phi
-void phi_gp_DIS(int *cov, double *Qeta1, double *det1, double *phi1, 
-     double *phis, int *phik, int *n, int *r, int *T, int *rT, int *N, 
+void phi_gp_DIS(int *cov, double *Qeta1, double *det1, double *phi1,  
+     double *phis, int *phik, double *nu, int *n, int *r, int *T, int *rT, int *N, 
      double *prior_a, double *prior_b, double *d,
      double *sig2eta, double *XB, double *o, int *constant, double *accept, 
      double *phip)
@@ -433,7 +457,7 @@ void phi_gp_DIS(int *cov, double *Qeta1, double *det1, double *phi1,
           
      for(i=0; i< *phik; i++){
         phitmp[0] = phis[i];
-        covFormat3(cov, n, phitmp, d, sig2eta, det, Qeta);
+        covFormat2(cov, n, phitmp, nu, d, sig2eta, det, Qeta);
         phidens_gp(phitmp, Qeta, det, n, r, T, rT, N, prior_a, prior_b, XB, 
         o, constant, out);
         pden[i] = out[0];
@@ -468,8 +492,7 @@ void phi_gp_DIS(int *cov, double *Qeta1, double *det1, double *phi1,
      tr1 = (double *) malloc((size_t)((col)*sizeof(double)));             
      phidens_gp(phi1, Qeta1, det1, n, r, T, rT, N, prior_a, prior_b, XB, 
      o, constant, tr1);
-
-     ratio[0] = exp(tr2[0] - tr1[0]);
+     ratio[0] = exp(tr2[0] + exp(tr2[0]) - tr1[0] - exp(tr1[0]));
      ratio_fnc(ratio, constant, U);
      if(U[0] < ratio[0]){
         phip[0] = phis[i];
@@ -532,7 +555,137 @@ void phidens_gp(double *phi, double *Qeta, double *det, int *n, int *r,
      return;
 }
 
+// Discrete sampling for nu
+void nu_gp_DIS(int *cov, double *Qeta1, double *det1, double *phi,  
+     double *nu1, int *n, int *r, int *T, int *rT, int *N, 
+     double *d,  double *sig2eta, double *XB, double *o, int *constant, 
+     double *nup)
+{
+    
+     int row, col, i, r1, T1, N1, rT1;
+     row = *n;
+     col = *constant;
+     r1 = *r;
+     T1 = *T;
+     N1 = row*r1*T1;
+     rT1 = *rT;
+     int nuk;
 
+     nuk=20;
+     double *nus;
+     nus = (double *) malloc((size_t)((nuk)*sizeof(double)));             
+     nus[0]=0.05; nus[1]=0.10; nus[2]=0.15; nus[3]=0.20; nus[4]=0.25;  
+     nus[5]=0.30; nus[6]=0.35; nus[7]=0.40; nus[8]=0.45; nus[9]=0.50;  
+     nus[10]=0.55; nus[11]=0.60; nus[12]=0.65; nus[13]=0.70; nus[14]=0.75; 
+     nus[15]=0.80; nus[16]=0.85; nus[17]=0.90; nus[18]=0.95; nus[19]=1.0; 
+/*
+     nuk=10;
+     double *nus;
+     nus = (double *) malloc((size_t)((nuk)*sizeof(double)));             
+     nus[0]=0.10; nus[1]=0.20; nus[2]=0.30; nus[3]=0.40; nus[4]=0.50;  
+     nus[5]=0.60; nus[6]=0.70; nus[7]=0.80; nus[8]=0.90; nus[9]=1.0;  
+*/          
+     double *nutmp, *pden, *Qeta, *det, *out;
+     nutmp = (double *) malloc((size_t)((col)*sizeof(double)));             
+     pden = (double *) malloc((size_t)((nuk)*sizeof(double)));             
+     Qeta = (double *) malloc((size_t)((row*row)*sizeof(double)));             
+     det = (double *) malloc((size_t)((col)*sizeof(double)));             
+     out = (double *) malloc((size_t)((col)*sizeof(double))); 
+     double u;
+     u =0.0;     
+          
+     for(i=0; i< nuk; i++){
+        nutmp[0] = nus[i];
+        covFormat2(cov, n, phi, nutmp, d, sig2eta, det, Qeta);
+        nudens_gp(Qeta, det, n, r, T, rT, N, XB, o, constant, out);
+        pden[i] = out[0];
+        u += out[0];
+     }     
+     free(nutmp); free(Qeta); free(det); free(out);
+     
+     double *pprob, *U, *tr2; 
+     pprob = (double *) malloc((size_t)((nuk)*sizeof(double)));             
+     U = (double *) malloc((size_t)((col)*sizeof(double)));
+     tr2 = (double *) malloc((size_t)((col)*sizeof(double)));             
+
+// cumulative prob
+     pprob[0] = pden[0]/u;         
+     for(i=0; i< (nuk-1); i++){
+        pprob[i+1] = pprob[i] + pden[i+1]/u;
+     }
+     runif_val(constant, constant, U);
+     if ( U[0] >  pprob[0]){
+     i = 0 ;
+     do{
+       i = i + 1;
+       } while ( ( U[0] > pprob[i] ) & ( i< nuk - 1 ) ) ;   
+     }
+     else i=0;
+//     nup[0] = nus[i];
+   
+     tr2[0] = pden[i];  
+     
+     free(pprob);
+     
+     double *ratio, *tr1;
+     ratio = (double *) malloc((size_t)((col)*sizeof(double)));             
+     tr1 = (double *) malloc((size_t)((col)*sizeof(double)));             
+     nudens_gp(Qeta1, det1, n, r, T, rT, N, XB, o, constant, tr1);
+     ratio[0] = exp(tr2[0] + exp(tr2[0]) - tr1[0] - exp(tr1[0]));
+     ratio_fnc(ratio, constant, U);
+     if(U[0] < ratio[0]){
+        nup[0] = nus[i];
+     }             
+     else {
+        nup[0] = nu1[0];
+     }     
+//     Rprintf("   i: %i, ratio: %4.4f, U: %4.4f, nup: %4.4f \n", i, ratio[i],U[0],nup[0]);
+      
+     free(ratio); free(tr2); free(tr1); free(pden); free(U);
+     free(nus);
+     
+     return;
+}     
+
+void nudens_gp(double *Qeta, double *det, int *n, int *r, int *T, int *rT, 
+     int *N, double *XB, double *o, int *constant, double *out)
+{
+     int row, col, l, i, j, r1, T1, N1, rT1;
+     row = *n;
+     col = *constant;
+     r1 = *r;
+     T1 = *T;
+     N1 = *N;
+     rT1 = *rT;
+
+     double *ov, *o1, *XB1; 
+     o1 = (double *) malloc((size_t)((row*col)*sizeof(double)));
+     ov = (double *) malloc((size_t)((row*col)*sizeof(double)));
+     XB1 = (double *) malloc((size_t)((row*col)*sizeof(double)));
+     double u;
+     u = 0.0;
+     for(l=0; l < r1; l++) {
+         for(i=0; i < T1; i++) {                                
+             extract_alt2(l, i, n, rT, T, o, o1);
+             extract_alt2(l, i, n, rT, T, XB, XB1);
+             for(j=0; j < row; j++) {
+                 ov[j]=o1[j]-XB1[j];
+             }       
+             u += xTay2(ov, Qeta, ov, row);
+         }
+     }
+     free(o1); free(ov); free(XB1); 
+
+     u =  0.5 * u;
+     if(det[0] <= 0){
+        det[0] = pow(1,-320);
+     }
+     double tr;
+     tr = 0.0;
+     tr = -0.5*rT1*log(det[0])-u; 
+     out[0] = tr;
+
+     return;
+}
 
 ////////////////////// THE END //////////////////////////
-
